-------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.tb_dp_pkg.ALL;

-- Purpose: Verify multiple variations of tb_dp_repack_data
-- Description:
-- Usage:
-- > as 6
-- > run -all

ENTITY tb_tb_dp_repack_data IS
END tb_tb_dp_repack_data;


ARCHITECTURE tb OF tb_tb_dp_repack_data IS

  CONSTANT c_nof_repeat  : NATURAL := 5;
  
  CONSTANT c_flow        : t_dp_flow_control_enum_arr := c_dp_flow_control_enum_arr;
  CONSTANT c_bool        : t_nat_boolean_arr := c_nat_boolean_arr;
  
BEGIN

  -- -- general
  -- g_flow_control_stimuli   : t_dp_flow_control_enum := e_active;  -- always e_active, e_random or e_pulse flow control
  -- g_flow_control_verify    : t_dp_flow_control_enum := e_active;  -- always e_active, e_random or e_pulse flow control
  -- -- specific
  -- g_in_dat_w               : NATURAL := 5;
  -- g_in_nof_words           : NATURAL := 2;
  -- g_pack_dat_w             : NATURAL := 16;
  -- g_pack_nof_words         : NATURAL := 1;
  -- g_in_bypass              : BOOLEAN := FALSE;  -- can use TRUE when g_in_nof_words=1  or g_in_nof_words=g_out_nof_words
  -- g_pack_bypass            : BOOLEAN := FALSE;  -- can use TRUE when g_out_nof_words=1 or g_in_nof_words=g_out_nof_words
  -- g_in_symbol_w            : NATURAL := 1;      -- default 1 for snk_in.empty  in nof bits, else use power of 2
  -- g_pack_symbol_w          : NATURAL := 1;      -- default 1 for src_out.empty in nof bits, else use power of 2
  -- g_nof_repeat             : NATURAL := 10;
  -- g_pkt_len                : NATURAL := 11;     -- if not a multiple of g_in_nof_words then the input stage flush creates gap between blocks
  -- g_pkt_gap                : NATURAL := 0
  
  g_flow_control_stimuli : FOR I IN 0 TO 2 GENERATE    -- 0 = e_active, 1 = e_random, 2 = e_pulse
    g_flow_control_verify : FOR J IN 0 TO 2 GENERATE    -- 0 = e_active, 1 = e_random, 2 = e_pulse

      -------------------------------------------------------------------------
      -- Tests that can use bypass
      -- . g_in_nof_words = 1
      -- . g_out_nof_words = 1
      -- . g_in_nof_words = g_in_nof_words >= 1
      -------------------------------------------------------------------------
      
      gen_bool_bypass : FOR K IN 0 TO 1 GENERATE
        gen_bool_bypass : FOR L IN 0 TO 1 GENERATE
          -- no repack, g_in_nof_words = g_out_nof_words = 1
          u_16_1_16_1_len_10_gap_0   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 16, 1, 16, 1, c_bool(K), c_bool(L), 1, 1, c_nof_repeat, 10, 0);  -- g_pkt_len > g_in_nof_words
          u_16_1_16_1_len_3_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 16, 1, 16, 1, c_bool(K), c_bool(L), 1, 1, c_nof_repeat,  3, 0);  -- g_pkt_len > g_in_nof_words, odd
          u_16_1_16_1_len_2_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 16, 1, 16, 1, c_bool(K), c_bool(L), 1, 1, c_nof_repeat,  2, 0);  -- g_pkt_len > g_in_nof_words, even
          u_16_1_16_1_len_1_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 16, 1, 16, 1, c_bool(K), c_bool(L), 1, 1, c_nof_repeat,  1, 0);  -- g_pkt_len = g_in_nof_words
          
          u_16_1_16_1_len_1_gap_1    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 16, 1, 16, 1, c_bool(K), c_bool(L), 1, 1, c_nof_repeat,  1, 1);  -- g_pkt_gap > 0
        END GENERATE;
      
        -- no repack, g_in_nof_words = g_out_nof_words > 1
        u_16_3_16_3_len_10_gap_0   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  16, 3, 16, 3, c_bool(K), c_bool(K), 1, 1, c_nof_repeat, 10,  0);
        
        -- g_in_nof_words > g_pack_nof_words can use always active stimuli except when g_pkt_len MOD g_in_nof_words /= 0, because then the input stage needs to flush
        u_8_4_32_1_len_1_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  8, 4, 32, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat,  1,  0);  -- g_pkt_len < g_in_nof_words
        u_8_4_32_1_len_2_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  8, 4, 32, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat,  2,  0);  -- g_pkt_len = g_in_nof_words
        u_8_4_32_1_len_3_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  8, 4, 32, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat,  3,  0);  -- g_pkt_len > g_in_nof_words, MOD /= 0
        u_8_4_32_1_len_10_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  8, 4, 32, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat, 10,  0);  -- g_pkt_len > g_in_nof_words, MOD /= 0
        u_8_4_32_1_len_11_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  8, 4, 32, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat, 11,  0);  -- g_pkt_len > g_in_nof_words, MOD /= 0
        u_8_4_32_1_len_12_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  8, 4, 32, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat, 12,  0);  -- g_pkt_len > g_in_nof_words, MOD = 0
      
        u_8_4_32_1_len_12_gap_2    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  8, 4, 32, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat, 12,  2);  -- g_pkt_gap > 0
      
        -- g_in_nof_words < g_pack_nof_words will apply backpressure, because the output stage needs to output more
        u_32_1_8_4_len_1_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 32, 1, 8, 4, c_bool(K), FALSE, 1, 1, c_nof_repeat,  1,  0);  -- g_pkt_len = g_in_nof_words
        u_32_1_8_4_len_2_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 32, 1, 8, 4, c_bool(K), FALSE, 1, 1, c_nof_repeat,  2,  0);  -- g_pkt_len > g_in_nof_words
        u_32_1_8_4_len_3_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 32, 1, 8, 4, c_bool(K), FALSE, 1, 1, c_nof_repeat,  3,  0);  -- g_pkt_len > g_in_nof_words
        u_32_1_8_4_len_10_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 32, 1, 8, 4, c_bool(K), FALSE, 1, 1, c_nof_repeat, 10,  0);  -- g_pkt_len > g_in_nof_words
        
        u_32_1_8_4_len_11_gap_1    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 32, 1, 8, 4, c_bool(K), FALSE, 1, 1, c_nof_repeat, 11,  1);  -- g_pkt_gap > 0
      
        -- g_in_dat_w MOD 8 /= 0, g_in_nof_words=1
        u_14_1_8_2_len_10_gap_0   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 14, 1, 8, 2, c_bool(K), FALSE, 1, 1, c_nof_repeat, 10,  0);  -- repack with subsection padding, even multiple of g_in_nof_words
        u_14_1_8_2_len_11_gap_0   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 14, 1, 8, 2, c_bool(K), FALSE, 1, 1, c_nof_repeat, 11,  0);  -- repack with subsection padding, odd multiple of g_in_nof_words
        
        -- g_in_dat_w MOD 8 /= 0, g_out_nof_words=1
        u_5_2_16_1_len_10_gap_0  : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 5, 2, 16, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat, 10,  0);  -- repack with subsection padding, integer multiple of g_in_nof_words
        u_5_2_16_1_len_11_gap_0  : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 5, 2, 16, 1, FALSE, c_bool(K), 1, 1, c_nof_repeat, 11,  0);  -- repack with subsection padding, fractional multiple of g_in_nof_words
        
        -- g_in_nof_words=1, g_pack_nof_words>1
        u_8_1_4_2_len_10_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   8, 1,  4,  2, c_bool(K), FALSE, 1, 1, c_nof_repeat, 10,  0);
        u_512_1_32_16_len_1_gap_20 : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 512, 1, 32, 16, c_bool(K), FALSE, 1, 1, c_nof_repeat,  1, 20);  -- pack a larger header slv into g_pack_dat_w words
        
        -- serialize to and deserialize from g_pack_dat_w=1 bit
        u_8_1_1_8_len_10_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  8, 1, 1,  8, c_bool(K), FALSE, 1, 1, c_nof_repeat, 10,  0);  -- g_pack_dat_w=1
        u_32_1_1_32_len_10_gap_7   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 32, 1, 1, 32, c_bool(K), FALSE, 1, 1, c_nof_repeat, 10,  7);  -- g_pack_dat_w=1
      
        -- g_in_symbol_w /= 1, g_out_symbol_w /= 1
        u_20_1_8_3_symbol_1_4_len_10_gap_0  : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 20, 1, 8, 3, c_bool(K), FALSE, 1, 4, c_nof_repeat, 10, 0);  -- no repack
        u_20_1_8_3_symbol_4_1_len_10_gap_0  : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 20, 1, 8, 3, c_bool(K), FALSE, 4, 1, c_nof_repeat, 10, 0);  -- no repack
        u_20_1_8_3_symbol_4_4_len_10_gap_0  : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 20, 1, 8, 3, c_bool(K), FALSE, 4, 4, c_nof_repeat, 10, 0);  -- no repack
        
        -- pack ETH/IP/UDP header, g_in_symbol_w = 8, g_out_symbol_w = 8
        u_336_1_32_11_symbol_8_8_len_1_gap_0  : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 336, 1, 32, 11, c_bool(K), FALSE, 8, 8, c_nof_repeat, 1, 0);  --pack to 32 bit --> empty = 2
        u_336_1_64_6_symbol_8_8_len_1_gap_0   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J), 336, 1, 64,  6, c_bool(K), FALSE, 8, 8, c_nof_repeat, 1, 0);  --pack to 64 bit --> empty = 6
      END GENERATE;
      
      -------------------------------------------------------------------------
      -- Tests that cannot use bypass
      -------------------------------------------------------------------------
      
      -- g_in_nof_words > 1 and g_pack_nof_words > 1
      u_24_2_16_3_len_1_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  24, 2, 16, 3, FALSE, FALSE, 1, 1, c_nof_repeat,  1, 0);  -- g_pkt_len < g_in_nof_words
      u_24_2_16_3_len_2_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  24, 2, 16, 3, FALSE, FALSE, 1, 1, c_nof_repeat,  2, 0);  -- g_pkt_len = g_in_nof_words
      u_24_2_16_3_len_3_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  24, 2, 16, 3, FALSE, FALSE, 1, 1, c_nof_repeat,  3, 0);  -- g_pkt_len = fractional multiple of g_in_nof_words
      u_24_2_16_3_len_10_gap_0   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  24, 2, 16, 3, FALSE, FALSE, 1, 1, c_nof_repeat, 10, 0);  -- g_pkt_len = integer multiple of g_in_nof_words
      u_24_2_16_3_len_11_gap_0   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  24, 2, 16, 3, FALSE, FALSE, 1, 1, c_nof_repeat, 11, 0);  -- g_pkt_len = fractional multiple of g_in_nof_words
      
      u_24_2_16_3_len_11_gap_3   : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  24, 2, 16, 3, FALSE, FALSE, 1, 1, c_nof_repeat, 11, 3);  -- g_pkt_gap > 0

      -- g_in_dat_w MOD 8 /= 0
      u_6_5_10_3_len_1_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   6, 5, 10, 3, FALSE, FALSE, 1, 1, c_nof_repeat,  1, 0);  -- g_pkt_len < g_in_nof_words
      u_6_5_10_3_len_2_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   6, 5, 10, 3, FALSE, FALSE, 1, 1, c_nof_repeat,  2, 0);  -- g_pkt_len < g_in_nof_words
      u_6_5_10_3_len_3_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   6, 5, 10, 3, FALSE, FALSE, 1, 1, c_nof_repeat,  3, 0);  -- g_pkt_len < g_in_nof_words
      u_6_5_10_3_len_4_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   6, 5, 10, 3, FALSE, FALSE, 1, 1, c_nof_repeat,  4, 0);  -- g_pkt_len < g_in_nof_words
      u_6_5_10_3_len_5_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   6, 5, 10, 3, FALSE, FALSE, 1, 1, c_nof_repeat,  5, 0);  -- g_pkt_len = g_in_nof_words
      u_6_5_10_3_len_10_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   6, 5, 10, 3, FALSE, FALSE, 1, 1, c_nof_repeat, 10, 0);  -- g_pkt_len = integer multiple of g_in_nof_words
      u_6_5_10_3_len_11_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   6, 5, 10, 3, FALSE, FALSE, 1, 1, c_nof_repeat, 11, 0);  -- g_pkt_len = fractional multiple of g_in_nof_words
      
      u_6_5_10_3_len_21_gap_3    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),   6, 5, 10, 3, FALSE, FALSE, 1, 1, c_nof_repeat, 21, 3);  -- g_pkt_gap > 0
      
      -- subsection padding, g_in_dat_w * g_in_nof_words < g_pack_dat_w * g_pack_nof_words
      u_18_2_8_5_len_1_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  18, 2,  8, 5, FALSE, FALSE, 1, 1, c_nof_repeat,  1, 0);  -- g_pkt_len < g_in_nof_words
      u_18_2_8_5_len_2_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  18, 2,  8, 5, FALSE, FALSE, 1, 1, c_nof_repeat,  2, 0);  -- g_pkt_len = g_in_nof_words
      u_18_2_8_5_len_3_gap_0     : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  18, 2,  8, 5, FALSE, FALSE, 1, 1, c_nof_repeat,  3, 0);  -- g_pkt_len = fractional multiple of g_in_nof_words
      u_18_2_8_5_len_10_gap_0    : ENTITY work.tb_dp_repack_data GENERIC MAP (c_flow(I), c_flow(J),  18, 2,  8, 5, FALSE, FALSE, 1, 1, c_nof_repeat, 10, 0);  -- g_pkt_len = integer multiple of g_in_nof_words
      
    END GENERATE;
  END GENERATE;
                                                                                               
END tb;
