-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Testbench running two tech_eth_10g with ppm offset.
-- Description:
--   The two instances of tb_ip_stratixiv_phy_xaui have a 100 ppm offset in their
--   tr_ref_clk_644.
--   The tb is self checking based on that tb_ip_stratixiv_phy_xaui is self checking
--   and both tb_ip_stratixiv_phy_xaui instances send the same and expect the same.
-- Remarks:
--   . The test fails when g_nof_10ppm /= 0 (erko, 4 dec 2014).
--   . It also appears that the serial data still has 320 ps period even when
--     g_nof_10ppm /=0 so probably the IP simulation model is not accurate enough
--     to verify ppm offsets.
-- Usage:
--   > do wave_tb_ip_stratixiv_phy_xaui_ppm.do
--   > run -all

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;

ENTITY tb_ip_stratixiv_phy_xaui_ppm IS
  GENERIC (
    g_sim_time   : TIME := 20 us;
    g_nof_10ppm  : INTEGER := 2    -- use /= 0 to verify XO ppm offset between two devices
  );
END tb_ip_stratixiv_phy_xaui_ppm;

ARCHITECTURE tb OF tb_ip_stratixiv_phy_xaui_ppm IS

  CONSTANT tr_clk_156_period : TIME := 6.4 ns;
  CONSTANT tr_clk_156_10ppm  : TIME := 64 fs;
  
  SIGNAL xaui_tx_serial_0 : STD_LOGIC_VECTOR(c_nof_xaui_lanes-1 DOWNTO 0);
  SIGNAL xaui_tx_serial_1 : STD_LOGIC_VECTOR(c_nof_xaui_lanes-1 DOWNTO 0);

BEGIN

  u_tb_ip_stratixiv_phy_xaui_0 : ENTITY work.tb_ip_stratixiv_phy_xaui
  GENERIC MAP (
    g_sim_time                => g_sim_time,
    g_tr_clk_156_period       => tr_clk_156_period,
    g_verify_link_recovery    => FALSE,
    g_use_xaui_rx_serial_in   => TRUE
  )
  PORT MAP (
    xaui_tx_serial_out  => xaui_tx_serial_0,
    xaui_rx_serial_in   => xaui_tx_serial_1
  );

  u_tb_ip_stratixiv_phy_xaui_1 : ENTITY work.tb_ip_stratixiv_phy_xaui
  GENERIC MAP (
    g_sim_time                => g_sim_time,
    g_tr_clk_156_period       => tr_clk_156_period + tr_clk_156_10ppm * g_nof_10ppm,
    g_verify_link_recovery    => FALSE,
    g_use_xaui_rx_serial_in   => TRUE
  )
  PORT MAP (
    xaui_tx_serial_out  => xaui_tx_serial_1,
    xaui_rx_serial_in   => xaui_tx_serial_0
  );
  
END tb;
