--------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, technology_lib, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;

ENTITY tech_xaui IS
  GENERIC (
    g_technology      : NATURAL := c_tech_select_default;
    g_sim             : BOOLEAN := FALSE;
    g_sim_level       : NATURAL := 0;     -- 0 = use IP; 1 = use fast serdes model
    g_nof_xaui        : NATURAL := 1 -- Up to 3 (hard XAUI only) supported
  );                      
  PORT (
    -- Transceiver PLL reference clock   
    tr_clk                    : IN  STD_LOGIC;  -- 156.25 MHz
    tr_rst                    : IN  STD_LOGIC;

    -- Calibration & reconfig clock
    cal_rec_clk               : IN  STD_LOGIC;

    -- MM interface 
    mm_clk                    : IN  STD_LOGIC;
    mm_rst                    : IN  STD_LOGIC;

    xaui_mosi                 : IN  t_mem_mosi := c_mem_mosi_rst;
    xaui_miso                 : OUT t_mem_miso;
    
    -- XGMII interface
    tx_clk_arr                : IN  STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);  -- 156.25 MHz, externally connect tr_clk to tx_clk_arr or connect rx_clk_arr to tx_clk_arr
    rx_clk_arr_out            : OUT STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);
    rx_clk_arr_in             : IN  STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);  -- externally connect to rx_clk_arr_out to avoid clock delta-delay
    
    txc_tx_ready_arr          : OUT STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);  -- tx_ready in tx_clk_arr domain, can be used for xon flow control
    rxc_rx_ready_arr          : OUT STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);  -- rx_ready in rx_clk_arr domain, typically leave not connected

    txc_rx_channelaligned_arr : OUT STD_LOGIC_VECTOR(g_nof_xaui-1 DOWNTO 0);  -- rx_channelaligned in tx_clk_arr domain, indicates that all 4 RX channels are aligned when asserted
    
    xgmii_tx_dc_arr           : IN  t_xgmii_dc_arr(g_nof_xaui-1 DOWNTO 0);
    xgmii_rx_dc_arr           : OUT t_xgmii_dc_arr(g_nof_xaui-1 DOWNTO 0);

    -- Serial I/O
    xaui_tx_arr               : OUT t_xaui_arr(g_nof_xaui-1 DOWNTO 0);
    xaui_rx_arr               : IN  t_xaui_arr(g_nof_xaui-1 DOWNTO 0)
  );
END tech_xaui;
          


ARCHITECTURE str OF tech_xaui IS  

  CONSTANT c_use_technology : BOOLEAN := g_sim = FALSE OR g_sim_level = 0;
  CONSTANT c_use_sim_model  : BOOLEAN := NOT c_use_technology;
  
BEGIN
 
  gen_ip_stratixiv : IF c_use_technology=TRUE AND g_technology=c_tech_stratixiv GENERATE
    u0 : ENTITY work.tech_xaui_stratixiv
    GENERIC MAP (g_sim, g_nof_xaui)
    PORT MAP (tr_clk, tr_rst, cal_rec_clk, mm_clk, mm_rst, xaui_mosi, xaui_miso,
              tx_clk_arr, rx_clk_arr_out, rx_clk_arr_in, txc_tx_ready_arr, rxc_rx_ready_arr, txc_rx_channelaligned_arr,
              xgmii_tx_dc_arr, xgmii_rx_dc_arr,
              xaui_tx_arr, xaui_rx_arr);
  END GENERATE;
  
  
  gem_sim_xaui : IF c_use_sim_model=TRUE GENERATE
    u0 : ENTITY work.sim_xaui
    GENERIC MAP (g_sim, g_nof_xaui)
    PORT MAP (tr_clk, tr_rst, cal_rec_clk, mm_clk, mm_rst, xaui_mosi, xaui_miso,
              tx_clk_arr, rx_clk_arr_out, rx_clk_arr_in, txc_tx_ready_arr, rxc_rx_ready_arr, txc_rx_channelaligned_arr,
              xgmii_tx_dc_arr, xgmii_rx_dc_arr,
              xaui_tx_arr, xaui_rx_arr);
  END GENERATE;
  
END str;

